@@ -1,5 +1,6 @@
 # Copyright (c) 2015-2018 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2017 Derek Gustafson <degustaf@gmail.com>
+# Copyright (c) 2019 Pierre Sassoulas <pierre.sassoulas@gmail.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -9,13 +10,10 @@
 import sys
 
 import astroid
-from astroid import bases
-from astroid import exceptions
+from astroid import bases, exceptions
 
-from pylint import checkers
+from pylint import checkers, interfaces, utils
 from pylint.checkers import utils as checker_utils
-from pylint import interfaces
-from pylint import utils
 from pylint.checkers.utils import decorated_with
 
 
