@@ -2,7 +2,7 @@
 # Copyright (c) 2006-2016 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2010 Daniel Harding <dharding@gmail.com>
 # Copyright (c) 2012-2014 Google, Inc.
-# Copyright (c) 2013-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2013-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Brett Cannon <brett@python.org>
 # Copyright (c) 2014 Arun Persaud <arun@nubati.net>
 # Copyright (c) 2015 Nick Bastin <nick.bastin@gmail.com>
@@ -13,24 +13,42 @@
 # Copyright (c) 2015 Florian Bruhin <me@the-compiler.org>
 # Copyright (c) 2015 Radu Ciorba <radu@devrandom.ro>
 # Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
+# Copyright (c) 2016, 2019 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2016, 2018 Jakub Wilk <jwilk@jwilk.net>
 # Copyright (c) 2016-2017 Łukasz Rogalski <rogalski.91@gmail.com>
 # Copyright (c) 2016 Glenn Matthews <glenn@e-dad.net>
 # Copyright (c) 2016 Elias Dorneles <eliasdorneles@gmail.com>
-# Copyright (c) 2016 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2016 Yannack <yannack@users.noreply.github.com>
 # Copyright (c) 2016 Alex Jurkiewicz <alex@jurkiewi.cz>
+# Copyright (c) 2017 Pierre Sassoulas <pierre.sassoulas@cea.fr>
+# Copyright (c) 2017, 2019 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2017 danields <danields761@gmail.com>
 # Copyright (c) 2017 Jacques Kvam <jwkvam@gmail.com>
 # Copyright (c) 2017 ttenhoeve-aa <ttenhoeve@appannie.com>
-# Copyright (c) 2017 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
+# Copyright (c) 2018-2019 Nick Drozd <nicholasdrozd@gmail.com>
+# Copyright (c) 2018-2019 Ville Skyttä <ville.skytta@iki.fi>
+# Copyright (c) 2018 Sergei Lebedev <185856+superbobry@users.noreply.github.com>
+# Copyright (c) 2018 Lucas Cimon <lucas.cimon@gmail.com>
+# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
+# Copyright (c) 2018 Natalie Serebryakova <natalie.serebryakova@Natalies-MacBook-Pro.local>
+# Copyright (c) 2018 Sushobhit <31987769+sushobhit27@users.noreply.github.com>
+# Copyright (c) 2018 SergeyKosarchuk <sergeykosarchuk@gmail.com>
 # Copyright (c) 2018 Steven M. Vascellaro <svascellaro@gmail.com>
 # Copyright (c) 2018 Mike Frysinger <vapier@gmail.com>
-# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
-# Copyright (c) 2018 Sushobhit <31987769+sushobhit27@users.noreply.github.com>
 # Copyright (c) 2018 Chris Lamb <chris@chris-lamb.co.uk>
 # Copyright (c) 2018 glmdgrielson <32415403+glmdgrielson@users.noreply.github.com>
-# Copyright (c) 2018 Ville Skyttä <ville.skytta@upcloud.com>
+# Copyright (c) 2019 Daniel Draper <Germandrummer92@users.noreply.github.com>
+# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
+# Copyright (c) 2019 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2019 Niko Wenselowski <niko@nerdno.de>
+# Copyright (c) 2019 Nikita Sobolev <mail@sobolevn.me>
+# Copyright (c) 2019 Oisín Moran <OisinMoran@users.noreply.github.com>
+# Copyright (c) 2019 Fantix King <fantix@uchicago.edu>
+# Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2020 bernie gray <bfgray3@users.noreply.github.com>
+# Copyright (c) 2020 Gabriel R Sezefredo <g@briel.dev>
+# Copyright (c) 2020 Benny <benny.mueller91@gmail.com>
+# Copyright (c) 2020 Anubhav <35621759+anubh-v@users.noreply.github.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -52,7 +70,11 @@ from astroid.arguments import CallSite
 import pylint.utils as lint_utils
 from pylint import checkers, exceptions, interfaces
 from pylint.checkers import utils
-from pylint.checkers.utils import is_property_setter_or_deleter
+from pylint.checkers.utils import (
+    is_overload_stub,
+    is_property_deleter,
+    is_property_setter,
+)
 from pylint.reporters.ureports import nodes as reporter_nodes
 
 
@@ -87,47 +109,47 @@ class NamingStyle:
 class SnakeCaseStyle(NamingStyle):
     """Regex rules for snake_case naming style."""
 
-    CLASS_NAME_RGX = re.compile("[a-z_][a-z0-9_]+$")
-    MOD_NAME_RGX = re.compile("([a-z_][a-z0-9_]*)$")
-    CONST_NAME_RGX = re.compile("(([a-z_][a-z0-9_]*)|(__.*__))$")
-    COMP_VAR_RGX = re.compile("[a-z_][a-z0-9_]*$")
+    CLASS_NAME_RGX = re.compile(r"[^\W\dA-Z][^\WA-Z]+$")
+    MOD_NAME_RGX = re.compile(r"[^\W\dA-Z][^\WA-Z]*$")
+    CONST_NAME_RGX = re.compile(r"([^\W\dA-Z][^\WA-Z]*|__.*__)$")
+    COMP_VAR_RGX = re.compile(r"[^\W\dA-Z][^\WA-Z]*$")
     DEFAULT_NAME_RGX = re.compile(
-        "(([a-z_][a-z0-9_]{2,})|(_[a-z0-9_]*)|(__[a-z][a-z0-9_]+__))$"
+        r"([^\W\dA-Z][^\WA-Z]{2,}|_[^\WA-Z]*|__[^\WA-Z\d_][^\WA-Z]+__)$"
     )
-    CLASS_ATTRIBUTE_RGX = re.compile(r"(([a-z_][a-z0-9_]{2,}|(__.*__)))$")
+    CLASS_ATTRIBUTE_RGX = re.compile(r"([^\W\dA-Z][^\WA-Z]{2,}|__.*__)$")
 
 
 class CamelCaseStyle(NamingStyle):
     """Regex rules for camelCase naming style."""
 
-    CLASS_NAME_RGX = re.compile("[a-z_][a-zA-Z0-9]+$")
-    MOD_NAME_RGX = re.compile("([a-z_][a-zA-Z0-9]*)$")
-    CONST_NAME_RGX = re.compile("(([a-z_][A-Za-z0-9]*)|(__.*__))$")
-    COMP_VAR_RGX = re.compile("[a-z_][A-Za-z0-9]*$")
-    DEFAULT_NAME_RGX = re.compile("(([a-z_][a-zA-Z0-9]{2,})|(__[a-z][a-zA-Z0-9_]+__))$")
-    CLASS_ATTRIBUTE_RGX = re.compile(r"([a-z_][A-Za-z0-9]{2,}|(__.*__))$")
+    CLASS_NAME_RGX = re.compile(r"[^\W\dA-Z][^\W_]+$")
+    MOD_NAME_RGX = re.compile(r"[^\W\dA-Z][^\W_]*$")
+    CONST_NAME_RGX = re.compile(r"([^\W\dA-Z][^\W_]*|__.*__)$")
+    COMP_VAR_RGX = re.compile(r"[^\W\dA-Z][^\W_]*$")
+    DEFAULT_NAME_RGX = re.compile(r"([^\W\dA-Z][^\W_]{2,}|__[^\W\dA-Z_]\w+__)$")
+    CLASS_ATTRIBUTE_RGX = re.compile(r"([^\W\dA-Z][^\W_]{2,}|__.*__)$")
 
 
 class PascalCaseStyle(NamingStyle):
     """Regex rules for PascalCase naming style."""
 
-    CLASS_NAME_RGX = re.compile("[A-Z_][a-zA-Z0-9]+$")
-    MOD_NAME_RGX = re.compile("[A-Z_][a-zA-Z0-9]+$")
-    CONST_NAME_RGX = re.compile("(([A-Z_][A-Za-z0-9]*)|(__.*__))$")
-    COMP_VAR_RGX = re.compile("[A-Z_][a-zA-Z0-9]+$")
-    DEFAULT_NAME_RGX = re.compile("[A-Z_][a-zA-Z0-9]{2,}$|(__[a-z][a-zA-Z0-9_]+__)$")
-    CLASS_ATTRIBUTE_RGX = re.compile("[A-Z_][a-zA-Z0-9]{2,}$")
+    CLASS_NAME_RGX = re.compile(r"[^\W\da-z][^\W_]+$")
+    MOD_NAME_RGX = re.compile(r"[^\W\da-z][^\W_]+$")
+    CONST_NAME_RGX = re.compile(r"([^\W\da-z][^\W_]*|__.*__)$")
+    COMP_VAR_RGX = re.compile(r"[^\W\da-z][^\W_]+$")
+    DEFAULT_NAME_RGX = re.compile(r"([^\W\da-z][^\W_]{2,}|__[^\W\dA-Z_]\w+__)$")
+    CLASS_ATTRIBUTE_RGX = re.compile(r"[^\W\da-z][^\W_]{2,}$")
 
 
 class UpperCaseStyle(NamingStyle):
     """Regex rules for UPPER_CASE naming style."""
 
-    CLASS_NAME_RGX = re.compile("[A-Z_][A-Z0-9_]+$")
-    MOD_NAME_RGX = re.compile("[A-Z_][A-Z0-9_]+$")
-    CONST_NAME_RGX = re.compile("(([A-Z_][A-Z0-9_]*)|(__.*__))$")
-    COMP_VAR_RGX = re.compile("[A-Z_][A-Z0-9_]+$")
-    DEFAULT_NAME_RGX = re.compile("([A-Z_][A-Z0-9_]{2,})|(__[a-z][a-zA-Z0-9_]+__)$")
-    CLASS_ATTRIBUTE_RGX = re.compile("[A-Z_][A-Z0-9_]{2,}$")
+    CLASS_NAME_RGX = re.compile(r"[^\W\da-z][^\Wa-z]+$")
+    MOD_NAME_RGX = re.compile(r"[^\W\da-z][^\Wa-z]+$")
+    CONST_NAME_RGX = re.compile(r"([^\W\da-z][^\Wa-z]*|__.*__)$")
+    COMP_VAR_RGX = re.compile(r"[^\W\da-z][^\Wa-z]+$")
+    DEFAULT_NAME_RGX = re.compile(r"([^\W\da-z][^\Wa-z]{2,}|__[^\W\dA-Z_]\w+__)$")
+    CLASS_ATTRIBUTE_RGX = re.compile(r"[^\W\da-z][^\Wa-z]{2,}$")
 
 
 class AnyStyle(NamingStyle):
@@ -149,7 +171,7 @@ NO_REQUIRED_DOC_RGX = re.compile("^_")
 REVERSED_PROTOCOL_METHOD = "__reversed__"
 SEQUENCE_PROTOCOL_METHODS = ("__getitem__", "__len__")
 REVERSED_METHODS = (SEQUENCE_PROTOCOL_METHODS, (REVERSED_PROTOCOL_METHOD,))
-TYPECHECK_COMPARISON_OPERATORS = frozenset(("is", "is not", "==", "!=", "in", "not in"))
+TYPECHECK_COMPARISON_OPERATORS = frozenset(("is", "is not", "==", "!="))
 LITERAL_NODE_TYPES = (astroid.Const, astroid.Dict, astroid.List, astroid.Set)
 UNITTEST_CASE = "unittest.case"
 BUILTINS = builtins.__name__
@@ -159,13 +181,25 @@ ABC_METACLASSES = {"_py_abc.ABCMeta", "abc.ABCMeta"}  # Python 3.7+,
 # Name categories that are always consistent with all naming conventions.
 EXEMPT_NAME_CATEGORIES = {"exempt", "ignore"}
 
-# A mapping from builtin-qname -> symbol, to be used when generating messages
+# A mapping from qname -> symbol, to be used when generating messages
 # about dangerous default values as arguments
 DEFAULT_ARGUMENT_SYMBOLS = dict(
     zip(
         [".".join([BUILTINS, x]) for x in ("set", "dict", "list")],
         ["set()", "{}", "[]"],
-    )
+    ),
+    **{
+        x: "%s()" % x
+        for x in (
+            "collections.deque",
+            "collections.ChainMap",
+            "collections.Counter",
+            "collections.OrderedDict",
+            "collections.defaultdict",
+            "collections.UserDict",
+            "collections.UserList",
+        )
+    },
 )
 REVERSED_COMPS = {"<": ">", "<=": ">=", ">": "<", ">=": "<="}
 COMPARISON_OPERATORS = frozenset(("==", "!=", "<", ">", "<=", ">="))
@@ -296,16 +330,15 @@ def _get_properties(config):
     if config is not None:
         property_classes.update(config.property_classes)
         property_names.update(
-            (prop.rsplit(".", 1)[-1] for prop in config.property_classes)
+            prop.rsplit(".", 1)[-1] for prop in config.property_classes
         )
     return property_classes, property_names
 
 
-def _determine_function_name_type(node, config=None):
+def _determine_function_name_type(node: astroid.FunctionDef, config=None):
     """Determine the name type whose regex the a function's name should match.
 
     :param node: A function node.
-    :type node: astroid.node_classes.NodeNG
     :param config: Configuration from which to pull additional property classes.
     :type config: :class:`optparse.Values`
 
@@ -316,7 +349,7 @@ def _determine_function_name_type(node, config=None):
     if not node.is_method():
         return "function"
 
-    if is_property_setter_or_deleter(node):
+    if is_property_setter(node) or is_property_deleter(node):
         # If the function is decorated using the prop_method.{setter,getter}
         # form, treat it like an attribute as well.
         return "attr"
@@ -333,7 +366,11 @@ def _determine_function_name_type(node, config=None):
             and decorator.attrname in property_names
         ):
             inferred = utils.safe_infer(decorator)
-            if inferred and inferred.qname() in property_classes:
+            if (
+                inferred
+                and hasattr(inferred, "qname")
+                and inferred.qname() in property_classes
+            ):
                 return "attr"
     return "method"
 
@@ -829,7 +866,6 @@ class BasicErrorChecker(_BasicChecker):
             node,
         )
         if defined_self is not node and not astroid.are_exclusive(node, defined_self):
-
             # Additional checks for methods which are not considered
             # redefined, since they are already part of the base API.
             if (
@@ -838,9 +874,33 @@ class BasicErrorChecker(_BasicChecker):
             ):
                 return
 
+            # Skip typing.overload() functions.
             if utils.is_overload_stub(node):
                 return
 
+            # Exempt functions redefined on a condition.
+            if isinstance(node.parent, astroid.If):
+                # Exempt "if not <func>" cases
+                if (
+                    isinstance(node.parent.test, astroid.UnaryOp)
+                    and node.parent.test.op == "not"
+                    and isinstance(node.parent.test.operand, astroid.Name)
+                    and node.parent.test.operand.name == node.name
+                ):
+                    return
+
+                # Exempt "if <func> is not None" cases
+                # pylint: disable=too-many-boolean-expressions
+                if (
+                    isinstance(node.parent.test, astroid.Compare)
+                    and isinstance(node.parent.test.left, astroid.Name)
+                    and node.parent.test.left.name == node.name
+                    and node.parent.test.ops[0][0] == "is"
+                    and isinstance(node.parent.test.ops[0][1], astroid.Const)
+                    and node.parent.test.ops[0][1].value is None
+                ):
+                    return
+
             # Check if we have forward references for this node.
             try:
                 redefinition_index = redefinitions.index(node)
@@ -1004,6 +1064,12 @@ class BasicChecker(_BasicChecker):
             'print("value: {}".format(123)). This might not be what the user '
             "intended to do.",
         ),
+        "W0129": (
+            "Assert statement has a string literal as its first argument. The assert will %s fail.",
+            "assert-on-string-literal",
+            "Used when an assert statement has a string literal as its first argument, which will "
+            "cause the assert to always pass.",
+        ),
     }
 
     reports = (("RP0101", "Statistics by type", report_by_type_stats),)
@@ -1247,7 +1313,10 @@ class BasicChecker(_BasicChecker):
     def _check_dangerous_default(self, node):
         # check for dangerous default values as arguments
         is_iterable = lambda n: isinstance(n, (astroid.List, astroid.Set, astroid.Dict))
-        for default in node.args.defaults:
+        defaults = node.args.defaults or [] + node.args.kw_defaults or []
+        for default in defaults:
+            if not default:
+                continue
             try:
                 value = next(default.infer())
             except astroid.InferenceError:
@@ -1365,9 +1434,9 @@ class BasicChecker(_BasicChecker):
                 elif name == "eval":
                     self.add_message("eval-used", node=node)
 
-    @utils.check_messages("assert-on-tuple")
+    @utils.check_messages("assert-on-tuple", "assert-on-string-literal")
     def visit_assert(self, node):
-        """check the use of an assert statement on a tuple."""
+        """check whether assert is used on a tuple or string literal."""
         if (
             node.fail is None
             and isinstance(node.test, astroid.Tuple)
@@ -1375,6 +1444,13 @@ class BasicChecker(_BasicChecker):
         ):
             self.add_message("assert-on-tuple", node=node)
 
+        if isinstance(node.test, astroid.Const) and isinstance(node.test.value, str):
+            if node.test.value:
+                when = "never"
+            else:
+                when = "always"
+            self.add_message("assert-on-string-literal", node=node, args=(when,))
+
     @utils.check_messages("duplicate-key")
     def visit_dict(self, node):
         """check duplicate key in dictionary"""
@@ -1479,7 +1555,7 @@ class BasicChecker(_BasicChecker):
 
     @utils.check_messages("confusing-with-statement")
     def visit_with(self, node):
-        # a "with" statement with multiple managers coresponds
+        # a "with" statement with multiple managers corresponds
         # to one AST "With" node with multiple items
         pairs = node.items
         if pairs:
@@ -1506,6 +1582,9 @@ class BasicChecker(_BasicChecker):
                 # A complex assignment, so bail out early.
                 return
             targets = targets[0].elts
+            if len(targets) == 1:
+                # Unpacking a variable into the same name.
+                return
 
         if isinstance(node.value, astroid.Name):
             if len(targets) != 1:
@@ -1532,6 +1611,10 @@ class BasicChecker(_BasicChecker):
                 )
 
     def _check_redeclared_assign_name(self, targets):
+        dummy_variables_rgx = lint_utils.get_global_option(
+            self, "dummy-variables-rgx", default=None
+        )
+
         for target in targets:
             if not isinstance(target, astroid.Tuple):
                 continue
@@ -1541,6 +1624,8 @@ class BasicChecker(_BasicChecker):
                 if isinstance(element, astroid.Tuple):
                     self._check_redeclared_assign_name([element])
                 elif isinstance(element, astroid.AssignName) and element.name != "_":
+                    if dummy_variables_rgx and dummy_variables_rgx.match(element.name):
+                        return
                     found_names.append(element.name)
 
             names = collections.Counter(found_names)
@@ -1649,6 +1734,11 @@ class NameChecker(_BasicChecker):
             "Used when the name doesn't conform to naming rules "
             "associated to its type (constant, variable, class...).",
         ),
+        "C0144": (
+            '%s name "%s" contains a non-ASCII unicode character',
+            "non-ascii-name",
+            "Used when the name contains at least one non-ASCII unicode character.",
+        ),
         "W0111": (
             "Name %s will become a keyword in Python %s",
             "assign-to-new-keyword",
@@ -1668,6 +1758,16 @@ class NameChecker(_BasicChecker):
                 " separated by a comma.",
             },
         ),
+        (
+            "good-names-rgxs",
+            {
+                "default": "",
+                "type": "regexp_csv",
+                "metavar": "<names>",
+                "help": "Good variable names regexes, separated by a comma. If names match any regex,"
+                " they will always be accepted",
+            },
+        ),
         (
             "bad-names",
             {
@@ -1678,6 +1778,16 @@ class NameChecker(_BasicChecker):
                 "separated by a comma.",
             },
         ),
+        (
+            "bad-names-rgxs",
+            {
+                "default": "",
+                "type": "regexp_csv",
+                "metavar": "<names>",
+                "help": "Bad variable names regexes, separated by a comma. If names match any regex,"
+                " they will always be refused",
+            },
+        ),
         (
             "name-group",
             {
@@ -1723,6 +1833,9 @@ class NameChecker(_BasicChecker):
         self._bad_names = {}
         self._name_regexps = {}
         self._name_hints = {}
+        self._good_names_rgxs_compiled = []
+        self._bad_names_rgxs_compiled = []
+        self._non_ascii_rgx_compiled = re.compile("[^\u0000-\u007F]")
 
     def open(self):
         self.stats = self.linter.add_stats(
@@ -1744,6 +1857,12 @@ class NameChecker(_BasicChecker):
         regexps, hints = self._create_naming_rules()
         self._name_regexps = regexps
         self._name_hints = hints
+        self._good_names_rgxs_compiled = [
+            re.compile(rgxp) for rgxp in self.config.good_names_rgxs
+        ]
+        self._bad_names_rgxs_compiled = [
+            re.compile(rgxp) for rgxp in self.config.bad_names_rgxs
+        ]
 
     def _create_naming_rules(self):
         regexps = {}
@@ -1767,7 +1886,7 @@ class NameChecker(_BasicChecker):
 
         return regexps, hints
 
-    @utils.check_messages("blacklisted-name", "invalid-name")
+    @utils.check_messages("blacklisted-name", "invalid-name", "non-ascii-name")
     def visit_module(self, node):
         self._check_name("module", node.name.split(".")[-1], node)
         self._bad_names = {}
@@ -1792,7 +1911,9 @@ class NameChecker(_BasicChecker):
             for args in warnings:
                 self._raise_name_warning(*args)
 
-    @utils.check_messages("blacklisted-name", "invalid-name", "assign-to-new-keyword")
+    @utils.check_messages(
+        "blacklisted-name", "invalid-name", "assign-to-new-keyword", "non-ascii-name"
+    )
     def visit_classdef(self, node):
         self._check_assign_to_new_keyword_violation(node.name, node)
         self._check_name("class", node.name, node)
@@ -1800,7 +1921,9 @@ class NameChecker(_BasicChecker):
             if not any(node.instance_attr_ancestors(attr)):
                 self._check_name("attr", attr, anodes[0])
 
-    @utils.check_messages("blacklisted-name", "invalid-name", "assign-to-new-keyword")
+    @utils.check_messages(
+        "blacklisted-name", "invalid-name", "assign-to-new-keyword", "non-ascii-name"
+    )
     def visit_functiondef(self, node):
         # Do not emit any warnings if the method is just an implementation
         # of a base class method.
@@ -1828,12 +1951,14 @@ class NameChecker(_BasicChecker):
 
     visit_asyncfunctiondef = visit_functiondef
 
-    @utils.check_messages("blacklisted-name", "invalid-name")
+    @utils.check_messages("blacklisted-name", "invalid-name", "non-ascii-name")
     def visit_global(self, node):
         for name in node.names:
             self._check_name("const", name, node)
 
-    @utils.check_messages("blacklisted-name", "invalid-name", "assign-to-new-keyword")
+    @utils.check_messages(
+        "blacklisted-name", "invalid-name", "assign-to-new-keyword", "non-ascii-name"
+    )
     def visit_assignname(self, node):
         """check module level assigned names"""
         self._check_assign_to_new_keyword_violation(node.name, node)
@@ -1842,14 +1967,15 @@ class NameChecker(_BasicChecker):
         if isinstance(assign_type, astroid.Comprehension):
             self._check_name("inlinevar", node.name, node)
         elif isinstance(frame, astroid.Module):
-            if isinstance(assign_type, astroid.Assign) and not in_loop(assign_type):
+            if isinstance(assign_type, astroid.Assign):
                 if isinstance(utils.safe_infer(assign_type.value), astroid.ClassDef):
                     self._check_name("class", node.name, node)
-                else:
-                    if not _redefines_import(node):
-                        # Don't emit if the name redefines an import
-                        # in an ImportError except handler.
-                        self._check_name("const", node.name, node)
+                # Don't emit if the name redefines an import
+                # in an ImportError except handler.
+                elif not _redefines_import(node) and isinstance(
+                    utils.safe_infer(assign_type.value), astroid.Const
+                ):
+                    self._check_name("const", node.name, node)
             elif isinstance(assign_type, astroid.ExceptHandler):
                 self._check_name("variable", node.name, node)
         elif isinstance(frame, astroid.FunctionDef):
@@ -1872,19 +1998,42 @@ class NameChecker(_BasicChecker):
     def _find_name_group(self, node_type):
         return self._name_group.get(node_type, node_type)
 
-    def _raise_name_warning(self, node, node_type, name, confidence):
+    def _raise_name_warning(
+        self, node, node_type, name, confidence, warning="invalid-name"
+    ):
         type_label = HUMAN_READABLE_TYPES[node_type]
         hint = self._name_hints[node_type]
         if self.config.include_naming_hint:
             hint += " (%r pattern)" % self._name_regexps[node_type].pattern
-        args = (type_label.capitalize(), name, hint)
+        args = (
+            (type_label.capitalize(), name, hint)
+            if warning == "invalid-name"
+            else (type_label.capitalize(), name)
+        )
 
-        self.add_message("invalid-name", node=node, args=args, confidence=confidence)
+        self.add_message(warning, node=node, args=args, confidence=confidence)
         self.stats["badname_" + node_type] += 1
 
+    def _name_valid_due_to_whitelist(self, name: str) -> bool:
+        return name in self.config.good_names or any(
+            pattern.match(name) for pattern in self._good_names_rgxs_compiled
+        )
+
+    def _name_invalid_due_to_blacklist(self, name: str) -> bool:
+        return name in self.config.bad_names or any(
+            pattern.match(name) for pattern in self._bad_names_rgxs_compiled
+        )
+
     def _check_name(self, node_type, name, node, confidence=interfaces.HIGH):
         """check for a name using the type's regexp"""
 
+        non_ascii_match = self._non_ascii_rgx_compiled.match(name)
+
+        if non_ascii_match is not None:
+            self._raise_name_warning(
+                node, node_type, name, confidence, warning="non-ascii-name"
+            )
+
         def _should_exempt_from_invalid_name(node):
             if node_type == "variable":
                 inferred = utils.safe_infer(node)
@@ -1896,9 +2045,9 @@ class NameChecker(_BasicChecker):
             clobbering, _ = utils.clobber_in_except(node)
             if clobbering:
                 return
-        if name in self.config.good_names:
+        if self._name_valid_due_to_whitelist(name=name):
             return
-        if name in self.config.bad_names:
+        if self._name_invalid_due_to_blacklist(name=name):
             self.stats["badname_" + node_type] += 1
             self.add_message("blacklisted-name", node=node, args=name)
             return
@@ -2013,7 +2162,11 @@ class DocStringChecker(_BasicChecker):
     def visit_functiondef(self, node):
         if self.config.no_docstring_rgx.match(node.name) is None:
             ftype = "method" if node.is_method() else "function"
-            if is_property_setter_or_deleter(node):
+            if (
+                is_property_setter(node)
+                or is_property_deleter(node)
+                or is_overload_stub(node)
+            ):
                 return
 
             if isinstance(node.parent.frame(), astroid.ClassDef):
@@ -2045,6 +2198,9 @@ class DocStringChecker(_BasicChecker):
     ):
         """check the node has a non empty docstring"""
         docstring = node.doc
+        if docstring is None:
+            docstring = _infer_dunder_doc_attribute(node)
+
         if docstring is None:
             if not report_missing:
                 return
@@ -2070,8 +2226,6 @@ class DocStringChecker(_BasicChecker):
                     func.bound, astroid.Instance
                 ):
                     # Strings.
-                    if func.bound.name == "str":
-                        return
                     if func.bound.name in ("str", "unicode", "bytes"):
                         return
             if node_type == "module":
@@ -2115,6 +2269,21 @@ def _is_one_arg_pos_call(call):
     return isinstance(call, astroid.Call) and len(call.args) == 1 and not call.keywords
 
 
+def _infer_dunder_doc_attribute(node):
+    # Try to see if we have a `__doc__` attribute.
+    try:
+        docstring = node["__doc__"]
+    except KeyError:
+        return None
+
+    docstring = utils.safe_infer(docstring)
+    if not docstring:
+        return None
+    if not isinstance(docstring, astroid.Const):
+        return None
+    return docstring.value
+
+
 class ComparisonChecker(_BasicChecker):
     """Checks for comparisons
 
