@@ -2,7 +2,7 @@
 # Copyright (c) 2006-2015 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2012-2014 Google, Inc.
 # Copyright (c) 2013 buck@yelp.com <buck@yelp.com>
-# Copyright (c) 2014-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Brett Cannon <brett@python.org>
 # Copyright (c) 2014 Arun Persaud <arun@nubati.net>
 # Copyright (c) 2015-2016 Moises Lopez <moylop260@vauxoo.com>
@@ -20,9 +20,19 @@
 # Copyright (c) 2017 Michka Popoff <michkapopoff@gmail.com>
 # Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
 # Copyright (c) 2017 Erik Wright <erik.wright@shopify.com>
+# Copyright (c) 2018 Lucas Cimon <lucas.cimon@gmail.com>
+# Copyright (c) 2018 Hornwitser <github@hornwitser.no>
+# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
+# Copyright (c) 2018 Natalie Serebryakova <natalie.serebryakova@Natalies-MacBook-Pro.local>
 # Copyright (c) 2018 Mike Frysinger <vapier@gmail.com>
 # Copyright (c) 2018 Sushobhit <31987769+sushobhit27@users.noreply.github.com>
 # Copyright (c) 2018 Marianna Polatoglou <mpolatoglou@bloomberg.net>
+# Copyright (c) 2019 Nick Drozd <nicholasdrozd@gmail.com>
+# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
+# Copyright (c) 2019 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2019 Nick Smith <clickthisnick@users.noreply.github.com>
+# Copyright (c) 2019 Paul Renvoisé <renvoisepaul@gmail.com>
+# Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -30,27 +40,27 @@
 """imports checkers for Python code"""
 
 import collections
-from distutils import sysconfig
+import copy
 import os
 import sys
-import copy
+from distutils import sysconfig
 
 import astroid
-from astroid import are_exclusive, decorators
-from astroid.modutils import get_module_part, is_standard_module
 import isort
+from astroid import modutils
+from astroid.decorators import cached
 
-from pylint.interfaces import IAstroidChecker
-from pylint.utils import get_global_option
-from pylint.exceptions import EmptyReportError
 from pylint.checkers import BaseChecker
 from pylint.checkers.utils import (
     check_messages,
-    node_ignores_exception,
     is_from_fallback_block,
+    node_ignores_exception,
 )
-from pylint.graph import get_cycles, DotBackend
-from pylint.reporters.ureports.nodes import VerbatimText, Paragraph
+from pylint.exceptions import EmptyReportError
+from pylint.graph import DotBackend, get_cycles
+from pylint.interfaces import IAstroidChecker
+from pylint.reporters.ureports.nodes import Paragraph, VerbatimText
+from pylint.utils import get_global_option
 
 
 def _qualified_names(modname):
@@ -114,7 +124,7 @@ def _get_first_import(node, context, name, base, level, alias):
                         break
                 if found:
                     break
-    if found and not are_exclusive(first, node):
+    if found and not astroid.are_exclusive(first, node):
         return first
     return None
 
@@ -200,7 +210,7 @@ MSGS = {
         "Unable to import %s",
         "import-error",
         "Used when pylint has been unable to import a module.",
-        {"old_names": [("F0401", "import-error")]},
+        {"old_names": [("F0401", "old-import-error")]},
     ),
     "E0402": (
         "Attempted relative import beyond top-level package",
@@ -223,12 +233,6 @@ MSGS = {
         "deprecated-module",
         "Used a module marked as deprecated is imported.",
     ),
-    "W0403": (
-        "Relative import %r, should be %r",
-        "relative-import",
-        "Used when an import relative to the package directory is detected.",
-        {"maxversion": (3, 0)},
-    ),
     "W0404": (
         "Reimport %r (imported line %s)",
         "reimported",
@@ -239,6 +243,11 @@ MSGS = {
         "import-self",
         "Used when a module is importing itself.",
     ),
+    "W0407": (
+        "Prefer importing %r instead of %r",
+        "preferred-module",
+        "Used when a module imported has a preferred replacement module.",
+    ),
     "W0410": (
         "__future__ import is not the first non docstring statement",
         "misplaced-future",
@@ -272,11 +281,18 @@ MSGS = {
         "Used when an import alias is same as original package."
         "e.g using import numpy as numpy instead of import numpy as np",
     ),
+    "C0415": (
+        "Import outside toplevel (%s)",
+        "import-outside-toplevel",
+        "Used when an import statement is used anywhere other than the module "
+        "toplevel. Move this import to the top of the file.",
+    ),
 }
 
 
 DEFAULT_STANDARD_LIBRARY = ()
 DEFAULT_KNOWN_THIRD_PARTY = ("enchant",)
+DEFAULT_PREFERRED_MODULES = ()
 
 
 class ImportsChecker(BaseChecker):
@@ -285,6 +301,7 @@ class ImportsChecker(BaseChecker):
     * relative / wildcard imports
     * cyclic imports
     * uses of deprecated modules
+    * uses of modules instead of preferred modules
     """
 
     __implements__ = IAstroidChecker
@@ -292,11 +309,8 @@ class ImportsChecker(BaseChecker):
     name = "imports"
     msgs = MSGS
     priority = -2
+    deprecated_modules = ("optparse", "tkinter.tix")
 
-    if sys.version_info < (3, 5):
-        deprecated_modules = ("optparse",)
-    else:
-        deprecated_modules = ("optparse", "tkinter.tix")
     options = (
         (
             "deprecated-modules",
@@ -308,6 +322,16 @@ class ImportsChecker(BaseChecker):
                 " separated by a comma.",
             },
         ),
+        (
+            "preferred-modules",
+            {
+                "default": DEFAULT_PREFERRED_MODULES,
+                "type": "csv",
+                "metavar": "<module:preferred-module>",
+                "help": "Couples of modules and preferred modules,"
+                " separated by a comma.",
+            },
+        ),
         (
             "import-graph",
             {
@@ -359,6 +383,18 @@ class ImportsChecker(BaseChecker):
                 "a third party library.",
             },
         ),
+        (
+            "allow-any-import-level",
+            {
+                "default": (),
+                "type": "csv",
+                "metavar": "<modules>",
+                "help": (
+                    "List of modules that can be imported at any level, not just "
+                    "the top level one."
+                ),
+            },
+        ),
         (
             "analyse-fallback-blocks",
             {
@@ -390,6 +426,7 @@ class ImportsChecker(BaseChecker):
         self._imports_stack = []
         self._first_non_import_node = None
         self._module_pkg = {}  # mapping of modules to the pkg they belong in
+        self._allow_any_import_level = set()
         self.reports = (
             ("RP0401", "External dependencies", self._report_external_dependencies),
             ("RP0402", "Modules dependencies graph", self._report_dependencies_graph),
@@ -431,6 +468,13 @@ class ImportsChecker(BaseChecker):
         self._module_pkg = {}  # mapping of modules to the pkg they belong in
         self._excluded_edges = collections.defaultdict(set)
         self._ignored_modules = get_global_option(self, "ignored-modules", default=[])
+        # Build a mapping {'module': 'preferred-module'}
+        self.preferred_modules = dict(
+            module.split(":")
+            for module in self.config.preferred_modules
+            if ":" in module
+        )
+        self._allow_any_import_level = set(self.config.allow_any_import_level)
 
     def _import_graph_without_ignored_edges(self):
         filtered_graph = copy.deepcopy(self.import_graph)
@@ -451,14 +495,15 @@ class ImportsChecker(BaseChecker):
         """triggered when an import statement is seen"""
         self._check_reimport(node)
         self._check_import_as_rename(node)
+        self._check_toplevel(node)
 
-        modnode = node.root()
         names = [name for name, _ in node.names]
         if len(names) >= 2:
             self.add_message("multiple-imports", args=", ".join(names), node=node)
 
         for name in names:
             self._check_deprecated_module(node, name)
+            self._check_preferred_module(node, name)
             imported_module = self._get_imported_module(node, name)
             if isinstance(node.parent, astroid.Module):
                 # Allow imports nested
@@ -469,7 +514,6 @@ class ImportsChecker(BaseChecker):
             if imported_module is None:
                 continue
 
-            self._check_relative_import(modnode, node, imported_module, name)
             self._add_imported_module(node, imported_module.name)
 
     @check_messages(*MSGS)
@@ -481,9 +525,11 @@ class ImportsChecker(BaseChecker):
         self._check_import_as_rename(node)
         self._check_misplaced_future(node)
         self._check_deprecated_module(node, basename)
+        self._check_preferred_module(node, basename)
         self._check_wildcard_imports(node, imported_module)
         self._check_same_line_imports(node)
         self._check_reimport(node, basename=basename, level=node.level)
+        self._check_toplevel(node)
 
         if isinstance(node.parent, astroid.Module):
             # Allow imports nested
@@ -492,9 +538,6 @@ class ImportsChecker(BaseChecker):
             self._record_import(node, imported_module)
         if imported_module is None:
             return
-        modnode = node.root()
-        self._check_relative_import(modnode, node, imported_module, basename)
-
         for name, _ in node.names:
             if name != "*":
                 self._add_imported_module(node, "%s.%s" % (imported_module.name, name))
@@ -506,14 +549,19 @@ class ImportsChecker(BaseChecker):
         # Check imports are grouped by category (standard, 3rd party, local)
         std_imports, ext_imports, loc_imports = self._check_imports_order(node)
 
-        # Check imports are grouped by package within a given category
-        met = set()
+        # Check that imports are grouped by package within a given category
+        met_import = set()  #  set for 'import x' style
+        met_from = set()  #  set for 'from x import y' style
         current_package = None
         for import_node, import_name in std_imports + ext_imports + loc_imports:
             if not self.linter.is_message_enabled(
                 "ungrouped-imports", import_node.fromlineno
             ):
                 continue
+            if isinstance(import_node, astroid.node_classes.ImportFrom):
+                met = met_from
+            else:
+                met = met_import
             package, _, _ = import_name.partition(".")
             if current_package and current_package != package and package in met:
                 self.add_message("ungrouped-imports", node=import_node, args=package)
@@ -737,7 +785,6 @@ class ImportsChecker(BaseChecker):
         except astroid.TooManyLevelsError:
             if _ignore_import_failure(importnode, modname, self._ignored_modules):
                 return None
-
             self.add_message("relative-beyond-top-level", node=importnode)
         except astroid.AstroidSyntaxError as exc:
             message = "Cannot import {!r} due to syntax error {!r}".format(
@@ -758,30 +805,6 @@ class ImportsChecker(BaseChecker):
             dotted_modname = _get_import_name(importnode, modname)
             self.add_message("import-error", args=repr(dotted_modname), node=importnode)
 
-    def _check_relative_import(
-        self, modnode, importnode, importedmodnode, importedasname
-    ):
-        """check relative import. node is either an Import or From node, modname
-        the imported module name.
-        """
-        if not self.linter.is_message_enabled("relative-import"):
-            return None
-        if importedmodnode.file is None:
-            return False  # built-in module
-        if modnode is importedmodnode:
-            return False  # module importing itself
-        if modnode.absolute_import_activated() or getattr(importnode, "level", None):
-            return False
-        if importedmodnode.name != importedasname:
-            # this must be a relative import...
-            self.add_message(
-                "relative-import",
-                args=(importedasname, importedmodnode.name),
-                node=importnode,
-            )
-            return None
-        return None
-
     def _add_imported_module(self, node, importedmodname):
         """notify an imported module, used to analyze dependencies"""
         module_file = node.root().file
@@ -789,14 +812,14 @@ class ImportsChecker(BaseChecker):
         base = os.path.splitext(os.path.basename(module_file))[0]
 
         try:
-            importedmodname = get_module_part(importedmodname, module_file)
+            importedmodname = modutils.get_module_part(importedmodname, module_file)
         except ImportError:
             pass
 
         if context_name == importedmodname:
             self.add_message("import-self", node=node)
 
-        elif not is_standard_module(importedmodname):
+        elif not modutils.is_standard_module(importedmodname):
             # if this is not a package __init__ module
             if base != "__init__" and context_name not in self._module_pkg:
                 # record the module's parent, or the module itself if this is
@@ -821,6 +844,15 @@ class ImportsChecker(BaseChecker):
             if mod_path == mod_name or mod_path.startswith(mod_name + "."):
                 self.add_message("deprecated-module", node=node, args=mod_path)
 
+    def _check_preferred_module(self, node, mod_path):
+        """check if the module has a preferred replacement"""
+        if mod_path in self.preferred_modules:
+            self.add_message(
+                "preferred-module",
+                node=node,
+                args=(self.preferred_modules[mod_path], mod_path),
+            )
+
     def _check_import_as_rename(self, node):
         names = node.names
         for name in names:
@@ -887,7 +919,7 @@ class ImportsChecker(BaseChecker):
             _make_graph(filename, self._internal_dependencies_info(), sect, "internal ")
 
     def _filter_dependencies_graph(self, internal):
-        """build the internal or the external depedency graph"""
+        """build the internal or the external dependency graph"""
         graph = collections.defaultdict(set)
         for importee, importers in self.stats["dependencies"].items():
             for importer in importers:
@@ -897,14 +929,14 @@ class ImportsChecker(BaseChecker):
                     graph[importee].add(importer)
         return graph
 
-    @decorators.cached
+    @cached
     def _external_dependencies_info(self):
         """return cached external dependencies information or build and
         cache them
         """
         return self._filter_dependencies_graph(internal=False)
 
-    @decorators.cached
+    @cached
     def _internal_dependencies_info(self):
         """return cached internal dependencies information or build and
         cache them
@@ -928,6 +960,31 @@ class ImportsChecker(BaseChecker):
             and "__all__" in imported_module.locals
         )
 
+    def _check_toplevel(self, node):
+        """Check whether the import is made outside the module toplevel.
+        """
+        # If the scope of the import is a module, then obviously it is
+        # not outside the module toplevel.
+        if isinstance(node.scope(), astroid.Module):
+            return
+
+        module_names = [
+            "{}.{}".format(node.modname, name[0])
+            if isinstance(node, astroid.ImportFrom)
+            else name[0]
+            for name in node.names
+        ]
+
+        # Get the full names of all the imports that are not whitelisted.
+        scoped_imports = [
+            name for name in module_names if name not in self._allow_any_import_level
+        ]
+
+        if scoped_imports:
+            self.add_message(
+                "import-outside-toplevel", args=", ".join(scoped_imports), node=node
+            )
+
 
 def register(linter):
     """required method to auto register this checker """
