@@ -4,7 +4,7 @@
 # Copyright (c) 2010 Daniel Harding <dharding@gmail.com>
 # Copyright (c) 2011-2014, 2017 Google, Inc.
 # Copyright (c) 2012 FELD Boris <lothiraldan@gmail.com>
-# Copyright (c) 2013-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2013-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Michal Nowikowski <godfryd@gmail.com>
 # Copyright (c) 2014 Brett Cannon <brett@python.org>
 # Copyright (c) 2014 Ricardo Gemignani <ricardo.gemignani@gmail.com>
@@ -13,21 +13,32 @@
 # Copyright (c) 2015 Radu Ciorba <radu@devrandom.ro>
 # Copyright (c) 2015 Simu Toni <simutoni@gmail.com>
 # Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
-# Copyright (c) 2016, 2018 Ashley Whetter <ashley@awhetter.co.uk>
+# Copyright (c) 2016, 2018-2019 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2016, 2018 Jakub Wilk <jwilk@jwilk.net>
 # Copyright (c) 2016-2017 Derek Gustafson <degustaf@gmail.com>
 # Copyright (c) 2016-2017 Łukasz Rogalski <rogalski.91@gmail.com>
 # Copyright (c) 2016 Grant Welch <gwelch925+github@gmail.com>
-# Copyright (c) 2017 Ville Skyttä <ville.skytta@iki.fi>
+# Copyright (c) 2017-2018 Ville Skyttä <ville.skytta@iki.fi>
 # Copyright (c) 2017-2018 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2017 Dan Garrette <dhgarrette@gmail.com>
+# Copyright (c) 2018-2019 Jim Robertson <jrobertson98atx@gmail.com>
+# Copyright (c) 2018 Mike Miller <mtmiller@users.noreply.github.com>
+# Copyright (c) 2018 Lucas Cimon <lucas.cimon@gmail.com>
+# Copyright (c) 2018 Drew <drewrisinger@users.noreply.github.com>
+# Copyright (c) 2018 Sushobhit <31987769+sushobhit27@users.noreply.github.com>
+# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
 # Copyright (c) 2018 Bryce Guinta <bryce.guinta@protonmail.com>
 # Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
 # Copyright (c) 2018 Mike Frysinger <vapier@gmail.com>
-# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
 # Copyright (c) 2018 Marianna Polatoglou <mpolatoglou@bloomberg.net>
 # Copyright (c) 2018 mar-chi-pan <mar.polatoglou@gmail.com>
-# Copyright (c) 2018 Ville Skyttä <ville.skytta@upcloud.com>
+# Copyright (c) 2019 Nick Drozd <nicholasdrozd@gmail.com>
+# Copyright (c) 2019 Djailla <bastien.vallet@gmail.com>
+# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
+# Copyright (c) 2019 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2020 Andrew Simmons <a.simmons@deakin.edu.au>
+# Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2020 Ashley Whetter <ashleyw@activestate.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -115,6 +126,7 @@ TYPING_NAMES = frozenset(
         "AnyStr",
         "Text",
         "Pattern",
+        "BinaryIO",
     }
 )
 
@@ -545,7 +557,7 @@ class NamesConsumer:
         del self.to_consume[name]
 
     def get_next_to_consume(self, node):
-        # mark the name as consumed if it's defined in this scope
+        # Get the definition of `node` from this scope
         name = node.name
         parent_node = node.parent
         found_node = self.to_consume.get(name)
@@ -557,6 +569,14 @@ class NamesConsumer:
             lhs = found_node[0].parent.targets[0]
             if lhs.name == name:  # this name is defined in this very statement
                 found_node = None
+
+        if (
+            found_node
+            and isinstance(parent_node, astroid.For)
+            and parent_node.iter == node
+            and parent_node.target in found_node
+        ):
+            found_node = None
         return found_node
 
 
@@ -659,8 +679,8 @@ class VariablesChecker(BaseChecker):
     def __init__(self, linter=None):
         BaseChecker.__init__(self, linter)
         self._to_consume = (
-            None
-        )  # list of tuples: (to_consume:dict, consumed:dict, scope_type:str)
+            None  # list of tuples: (to_consume:dict, consumed:dict, scope_type:str)
+        )
         self._checking_mod_attr = None
         self._loop_variables = []
         self._type_annotation_names = []
@@ -928,11 +948,8 @@ class VariablesChecker(BaseChecker):
     def visit_delname(self, node):
         self.visit_name(node)
 
-    @utils.check_messages(*MSGS)
     def visit_name(self, node):
-        """check that a name is defined if the current scope and doesn't
-        redefine a built-in
-        """
+        """Check that a name is defined in the current scope"""
         stmt = node.statement()
         if stmt.fromlineno is None:
             # name node from an astroid built from live code, skip
@@ -952,6 +969,14 @@ class VariablesChecker(BaseChecker):
             start_index = len(self._to_consume) - 2
         else:
             start_index = len(self._to_consume) - 1
+
+        undefined_variable_is_enabled = self.linter.is_message_enabled(
+            "undefined-variable"
+        )
+        used_before_assignment_is_enabled = self.linter.is_message_enabled(
+            "used-before-assignment"
+        )
+
         # iterates through parent scopes, from the inner to the outer
         base_scope_type = self._to_consume[start_index].scope_type
         # pylint: disable=too-many-nested-blocks; refactoring this block is a pain.
@@ -960,13 +985,11 @@ class VariablesChecker(BaseChecker):
             # if the current scope is a class scope but it's not the inner
             # scope, ignore it. This prevents to access this scope instead of
             # the globals one in function members when there are some common
-            # names. The only exception is when the starting scope is a
-            # comprehension and its direct outer scope is a class
-            if (
-                current_consumer.scope_type == "class"
-                and i != start_index
-                and not (base_scope_type == "comprehension" and i == start_index - 1)
-            ):
+            # names.
+            if current_consumer.scope_type == "class" and i != start_index:
+                # The only exceptions are: when the variable forms an iter within a
+                # comprehension scope; and/or when used as a default, decorator,
+                # or annotation within a function.
                 if self._ignore_class_scope(node):
                     continue
 
@@ -990,7 +1013,9 @@ class VariablesChecker(BaseChecker):
             # checks for use before assignment
             defnode = utils.assign_parent(current_consumer.to_consume[name][0])
 
-            if defnode is not None:
+            if (
+                undefined_variable_is_enabled or used_before_assignment_is_enabled
+            ) and defnode is not None:
                 self._check_late_binding_closure(node, defnode)
                 defstmt = defnode.statement()
                 defframe = defstmt.frame()
@@ -1029,7 +1054,11 @@ class VariablesChecker(BaseChecker):
                     # -- quit
                     break
 
-                maybee0601, annotation_return, use_outer_definition = self._is_variable_violation(
+                (
+                    maybee0601,
+                    annotation_return,
+                    use_outer_definition,
+                ) = self._is_variable_violation(
                     node,
                     name,
                     defnode,
@@ -1121,10 +1150,15 @@ class VariablesChecker(BaseChecker):
         else:
             # we have not found the name, if it isn't a builtin, that's an
             # undefined name !
-            if not (
+            if undefined_variable_is_enabled and not (
                 name in astroid.Module.scope_attrs
                 or utils.is_builtin(name)
                 or name in self.config.additional_builtins
+                or (
+                    name == "__class__"
+                    and isinstance(frame, astroid.FunctionDef)
+                    and frame.is_method()
+                )
             ):
                 if not utils.node_ignores_exception(node, NameError):
                     self.add_message("undefined-variable", args=name, node=node)
@@ -1143,6 +1177,8 @@ class VariablesChecker(BaseChecker):
                 module = next(_infer_name_module(node, parts[0]))
             except astroid.ResolveError:
                 continue
+            if not isinstance(module, astroid.Module):
+                continue
             self._check_module_attrs(node, module, parts[1:])
 
     @utils.check_messages("no-name-in-module")
@@ -1223,15 +1259,53 @@ class VariablesChecker(BaseChecker):
 
     @staticmethod
     def _defined_in_function_definition(node, frame):
-        in_annotation_or_default = False
+        in_annotation_or_default_or_decorator = False
         if isinstance(frame, astroid.FunctionDef) and node.statement() is frame:
-            in_annotation_or_default = (
-                node in frame.args.annotations
-                or node in frame.args.kwonlyargs_annotations
-                or node is frame.args.varargannotation
-                or node is frame.args.kwargannotation
-            ) or frame.args.parent_of(node)
-        return in_annotation_or_default
+            in_annotation_or_default_or_decorator = (
+                (
+                    node in frame.args.annotations
+                    or node in frame.args.posonlyargs_annotations
+                    or node in frame.args.kwonlyargs_annotations
+                    or node is frame.args.varargannotation
+                    or node is frame.args.kwargannotation
+                )
+                or frame.args.parent_of(node)
+                or (frame.decorators and frame.decorators.parent_of(node))
+                or (
+                    frame.returns
+                    and (node is frame.returns or frame.returns.parent_of(node))
+                )
+            )
+        return in_annotation_or_default_or_decorator
+
+    @staticmethod
+    def _in_lambda_or_comprehension_body(
+        node: astroid.node_classes.NodeNG, frame: astroid.node_classes.NodeNG
+    ) -> bool:
+        """return True if node within a lambda/comprehension body (or similar) and thus should not have access to class attributes in frame"""
+        child = node
+        parent = node.parent
+        while parent is not None:
+            if parent is frame:
+                return False
+            if isinstance(parent, astroid.Lambda) and not child is parent.args:
+                # Body of lambda should not have access to class attributes.
+                return True
+            if (
+                isinstance(parent, astroid.node_classes.Comprehension)
+                and not child is parent.iter
+            ):
+                # Only iter of list/set/dict/generator comprehension should have access.
+                return True
+            if isinstance(parent, astroid.scoped_nodes.ComprehensionScope) and not (
+                parent.generators and child is parent.generators[0]
+            ):
+                # Body of list/set/dict/generator comprehension should not have access to class attributes.
+                # Furthermore, only the first generator (if multiple) in comprehension should have access.
+                return True
+            child = parent
+            parent = parent.parent
+        return False
 
     @staticmethod
     def _is_variable_violation(
@@ -1275,13 +1349,12 @@ class VariablesChecker(BaseChecker):
                 use_outer_definition = stmt == defstmt and not isinstance(
                     defnode, astroid.node_classes.Comprehension
                 )
-            else:
-                # check if we have a nonlocal
-                if name in defframe.locals:
-                    maybee0601 = not any(
-                        isinstance(child, astroid.Nonlocal) and name in child.names
-                        for child in defframe.get_children()
-                    )
+            # check if we have a nonlocal
+            elif name in defframe.locals:
+                maybee0601 = not any(
+                    isinstance(child, astroid.Nonlocal) and name in child.names
+                    for child in defframe.get_children()
+                )
 
         if (
             base_scope_type == "lambda"
@@ -1393,13 +1466,19 @@ class VariablesChecker(BaseChecker):
 
         name = node.name
         frame = node.statement().scope()
-        in_annotation_or_default = self._defined_in_function_definition(node, frame)
-        if in_annotation_or_default:
+        in_annotation_or_default_or_decorator = self._defined_in_function_definition(
+            node, frame
+        )
+        if in_annotation_or_default_or_decorator:
             frame_locals = frame.parent.scope().locals
         else:
             frame_locals = frame.locals
         return not (
-            (isinstance(frame, astroid.ClassDef) or in_annotation_or_default)
+            (
+                isinstance(frame, astroid.ClassDef)
+                or in_annotation_or_default_or_decorator
+            )
+            and not self._in_lambda_or_comprehension_body(node, frame)
             and name in frame_locals
         )
 
@@ -1616,6 +1695,9 @@ class VariablesChecker(BaseChecker):
         self.add_message("unused-argument", args=name, node=stmt, confidence=confidence)
 
     def _check_late_binding_closure(self, node, assignment_node):
+        if not self.linter.is_message_enabled("cell-var-from-loop"):
+            return
+
         def _is_direct_lambda_call():
             return (
                 isinstance(node_scope.parent, astroid.Call)
@@ -1729,7 +1811,7 @@ class VariablesChecker(BaseChecker):
             if isinstance(target, astroid.node_classes.AssignName)
         )
         if self_cls_name in target_assign_names:
-            self.add_message("self-cls-assignment", node=node, args=(self_cls_name))
+            self.add_message("self-cls-assignment", node=node, args=(self_cls_name,))
 
     def _check_unpacking(self, inferred, node, targets):
         """ Check for unbalanced tuple unpacking
@@ -1765,20 +1847,18 @@ class VariablesChecker(BaseChecker):
                     ),
                 )
         # attempt to check unpacking may be possible (ie RHS is iterable)
-        else:
-            if not utils.is_iterable(inferred):
-                self.add_message(
-                    "unpacking-non-sequence",
-                    node=node,
-                    args=(_get_unpacking_extra_info(node, inferred),),
-                )
+        elif not utils.is_iterable(inferred):
+            self.add_message(
+                "unpacking-non-sequence",
+                node=node,
+                args=(_get_unpacking_extra_info(node, inferred),),
+            )
 
     def _check_module_attrs(self, node, module, module_names):
         """check that module_names (list of string) are accessible through the
         given module
         if the latest access name corresponds to a module, return it
         """
-        assert isinstance(module, astroid.Module), module
         while module_names:
             name = module_names.pop(0)
             if name == "__dict__":
@@ -1877,6 +1957,10 @@ class VariablesChecker(BaseChecker):
                     continue
                 checked.add(real_name)
 
+                is_type_annotation_import = (
+                    imported_name in self._type_annotation_names
+                    or as_name in self._type_annotation_names
+                )
                 if isinstance(stmt, astroid.Import) or (
                     isinstance(stmt, astroid.ImportFrom) and not stmt.modname
                 ):
@@ -1887,7 +1971,7 @@ class VariablesChecker(BaseChecker):
                         # because they can be imported for exporting.
                         continue
 
-                    if imported_name in self._type_annotation_names:
+                    if is_type_annotation_import:
                         # Most likely a typing import if it wasn't used so far.
                         continue
 
@@ -1910,7 +1994,7 @@ class VariablesChecker(BaseChecker):
                         # __future__ import in another module.
                         continue
 
-                    if imported_name in self._type_annotation_names:
+                    if is_type_annotation_import:
                         # Most likely a typing import if it wasn't used so far.
                         continue
 
@@ -1950,6 +2034,8 @@ class VariablesChecker(BaseChecker):
         name = None
         if isinstance(klass._metaclass, astroid.Name):
             name = klass._metaclass.name
+        elif isinstance(klass._metaclass, astroid.Attribute) and klass._metaclass.expr:
+            name = klass._metaclass.expr.name
         elif metaclass:
             name = metaclass.root().name
 
@@ -1967,8 +2053,11 @@ class VariablesChecker(BaseChecker):
             name = None
             if isinstance(klass._metaclass, astroid.Name):
                 name = klass._metaclass.name
-            elif isinstance(klass._metaclass, astroid.Attribute):
-                name = klass._metaclass.as_string()
+            elif (
+                isinstance(klass._metaclass, astroid.Attribute)
+                and klass._metaclass.expr
+            ):
+                name = klass._metaclass.expr.name
 
             if name is not None:
                 if not (
