@@ -4,7 +4,7 @@
 # Copyright (c) 2010 Julien Jehannet <julien.jehannet@logilab.fr>
 # Copyright (c) 2013 Google, Inc.
 # Copyright (c) 2013 John McGehee <jmcgehee@altera.com>
-# Copyright (c) 2014-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Brett Cannon <brett@python.org>
 # Copyright (c) 2014 Arun Persaud <arun@nubati.net>
 # Copyright (c) 2015 Aru Sahni <arusahni@gmail.com>
@@ -13,17 +13,22 @@
 # Copyright (c) 2016 Erik <erik.eriksson@yahoo.com>
 # Copyright (c) 2016 Alexander Todorov <atodorov@otb.bg>
 # Copyright (c) 2016 Moises Lopez <moylop260@vauxoo.com>
-# Copyright (c) 2017-2018 Ville Skyttä <ville.skytta@iki.fi>
+# Copyright (c) 2017-2019 Ville Skyttä <ville.skytta@iki.fi>
 # Copyright (c) 2017 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2017 ahirnish <ahirnish@gmail.com>
 # Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
-# Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
+# Copyright (c) 2018, 2020 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2018 Jim Robertson <jrobertson98atx@gmail.com>
 # Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
+# Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
 # Copyright (c) 2018 Sushobhit <31987769+sushobhit27@users.noreply.github.com>
-# Copyright (c) 2018 Anthony Sottile <asottile@umich.edu>
 # Copyright (c) 2018 Gary Tyler McLeod <mail@garytyler.com>
 # Copyright (c) 2018 Konstantin <Github@pheanex.de>
 # Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
+# Copyright (c) 2019 Janne Rönkkö <jannero@users.noreply.github.com>
+# Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
+# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
+# Copyright (c) 2019 Pierre Sassoulas <pierre.sassoulas@gmail.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -33,31 +38,23 @@
 * pylintrc
 * pylint.d (PYLINTHOME)
 """
-from __future__ import print_function
-
-# TODO(cpopa): this module contains the logic for the
-# configuration parser and for the command line parser,
-# but it's really coupled to optparse's internals.
-# The code was copied almost verbatim from logilab.common,
-# in order to not depend on it anymore and it will definitely
-# need a cleanup. It could be completely reengineered as well.
-
-import contextlib
 import collections
+import configparser
+import contextlib
 import copy
-import io
+import functools
 import optparse
 import os
 import pickle
 import re
 import sys
 import time
+from typing import Any, Dict, Tuple
 
-import configparser
+import toml
 
 from pylint import utils
 
-
 USER_HOME = os.path.expanduser("~")
 if "PYLINTHOME" in os.environ:
     PYLINT_HOME = os.environ["PYLINTHOME"]
@@ -77,18 +74,12 @@ def _get_pdata_path(base_name, recurs):
 def load_results(base):
     data_file = _get_pdata_path(base, 1)
     try:
-        with open(data_file, _PICK_LOAD) as stream:
+        with open(data_file, "rb") as stream:
             return pickle.load(stream)
     except Exception:  # pylint: disable=broad-except
         return {}
 
 
-if sys.version_info < (3, 0):
-    _PICK_DUMP, _PICK_LOAD = "w", "r"
-else:
-    _PICK_DUMP, _PICK_LOAD = "wb", "rb"
-
-
 def save_results(results, base):
     if not os.path.exists(PYLINT_HOME):
         try:
@@ -97,44 +88,76 @@ def save_results(results, base):
             print("Unable to create directory %s" % PYLINT_HOME, file=sys.stderr)
     data_file = _get_pdata_path(base, 1)
     try:
-        with open(data_file, _PICK_DUMP) as stream:
+        with open(data_file, "wb") as stream:
             pickle.dump(results, stream)
-    except (IOError, OSError) as ex:
+    except OSError as ex:
         print("Unable to create file %s: %s" % (data_file, ex), file=sys.stderr)
 
 
+def _toml_has_config(path):
+    with open(path) as toml_handle:
+        content = toml.load(toml_handle)
+        try:
+            content["tool"]["pylint"]
+        except KeyError:
+            return False
+
+    return True
+
+
+def _cfg_has_config(path):
+    parser = configparser.ConfigParser()
+    parser.read(path)
+    return any(section.startswith("pylint.") for section in parser.sections())
+
+
+def find_default_config_files():
+    """Find all possible config files."""
+    rc_names = ("pylintrc", ".pylintrc")
+    config_names = rc_names + ("pyproject.toml", "setup.cfg")
+    for config_name in config_names:
+        if os.path.isfile(config_name):
+            if config_name.endswith(".toml") and not _toml_has_config(config_name):
+                continue
+            if config_name.endswith(".cfg") and not _cfg_has_config(config_name):
+                continue
+
+            yield os.path.abspath(config_name)
+
+    if os.path.isfile("__init__.py"):
+        curdir = os.path.abspath(os.getcwd())
+        while os.path.isfile(os.path.join(curdir, "__init__.py")):
+            curdir = os.path.abspath(os.path.join(curdir, ".."))
+            for rc_name in rc_names:
+                rc_path = os.path.join(curdir, rc_name)
+                if os.path.isfile(rc_path):
+                    yield rc_path
+
+    if "PYLINTRC" in os.environ and os.path.exists(os.environ["PYLINTRC"]):
+        if os.path.isfile(os.environ["PYLINTRC"]):
+            yield os.environ["PYLINTRC"]
+    else:
+        user_home = os.path.expanduser("~")
+        if user_home not in ("~", "/root"):
+            home_rc = os.path.join(user_home, ".pylintrc")
+            if os.path.isfile(home_rc):
+                yield home_rc
+            home_rc = os.path.join(user_home, ".config", "pylintrc")
+            if os.path.isfile(home_rc):
+                yield home_rc
+
+    if os.path.isfile("/etc/pylintrc"):
+        yield "/etc/pylintrc"
+
+
 def find_pylintrc():
     """search the pylint rc file and return its path if it find it, else None
     """
-    # is there a pylint rc file in the current directory ?
-    if os.path.exists("pylintrc"):
-        return os.path.abspath("pylintrc")
-    if os.path.exists(".pylintrc"):
-        return os.path.abspath(".pylintrc")
-    if os.path.isfile("__init__.py"):
-        curdir = os.path.abspath(os.getcwd())
-        while os.path.isfile(os.path.join(curdir, "__init__.py")):
-            curdir = os.path.abspath(os.path.join(curdir, ".."))
-            if os.path.isfile(os.path.join(curdir, "pylintrc")):
-                return os.path.join(curdir, "pylintrc")
-            if os.path.isfile(os.path.join(curdir, ".pylintrc")):
-                return os.path.join(curdir, ".pylintrc")
-    if "PYLINTRC" in os.environ and os.path.exists(os.environ["PYLINTRC"]):
-        pylintrc = os.environ["PYLINTRC"]
-    else:
-        user_home = os.path.expanduser("~")
-        if user_home in ("~", "/root"):
-            pylintrc = ".pylintrc"
-        else:
-            pylintrc = os.path.join(user_home, ".pylintrc")
-            if not os.path.isfile(pylintrc):
-                pylintrc = os.path.join(user_home, ".config", "pylintrc")
-    if not os.path.isfile(pylintrc):
-        if os.path.isfile("/etc/pylintrc"):
-            pylintrc = "/etc/pylintrc"
-        else:
-            pylintrc = None
-    return pylintrc
+    for config_file in find_default_config_files():
+        if config_file.endswith("pylintrc"):
+            return config_file
+
+    return None
 
 
 PYLINTRC = find_pylintrc()
@@ -246,7 +269,6 @@ def _validate(value, optdict, name=""):
     try:
         _type = optdict["type"]
     except KeyError:
-        # FIXME
         return value
     return _call_validator(_type, optdict, name, value)
 
@@ -494,7 +516,7 @@ Please report bugs on the project\'s mailing list:
 .SH AUTHOR
 %s <%s>
 """ % (
-            getattr(pkginfo, "debian_name", pkginfo.modname),
+            getattr(pkginfo, "debian_name", "pylint"),
             pkginfo.mailinglist,
             pkginfo.author,
             pkginfo.author_email,
@@ -702,10 +724,8 @@ class OptionsManagerMixIn:
             opt = "-".join(["long"] * helplevel) + "-help"
             if opt in self._all_options:
                 break  # already processed
-            # pylint: disable=unused-argument
-            def helpfunc(option, opt, val, p, level=helplevel):
-                print(self.help(level))
-                sys.exit(0)
+
+            helpfunc = functools.partial(self.helpfunc, level=helplevel)
 
             helpmsg = "%s verbose help." % " ".join(["more"] * helplevel)
             optdict = {"action": "callback", "callback": helpfunc, "help": helpmsg}
@@ -718,20 +738,34 @@ class OptionsManagerMixIn:
         if config_file is not None:
             config_file = os.path.expanduser(config_file)
             if not os.path.exists(config_file):
-                raise IOError("The config file {:s} doesn't exist!".format(config_file))
+                raise OSError("The config file {:s} doesn't exist!".format(config_file))
 
         use_config_file = config_file and os.path.exists(config_file)
         if use_config_file:
             parser = self.cfgfile_parser
 
-            # Use this encoding in order to strip the BOM marker, if any.
-            with io.open(config_file, "r", encoding="utf_8_sig") as fp:
-                parser.read_file(fp)
+            if config_file.endswith(".toml"):
+                with open(config_file) as fp:
+                    content = toml.load(fp)
 
-            # normalize sections'title
-            for sect, values in list(parser._sections.items()):
-                if not sect.isupper() and values:
-                    parser._sections[sect.upper()] = values
+                try:
+                    sections_values = content["tool"]["pylint"]
+                except KeyError:
+                    pass
+                else:
+                    for section, values in sections_values.items():
+                        parser._sections[section.upper()] = values
+            else:
+                # Use this encoding in order to strip the BOM marker, if any.
+                with open(config_file, encoding="utf_8_sig") as fp:
+                    parser.read_file(fp)
+
+                # normalize sections'title
+                for sect, values in list(parser._sections.items()):
+                    if sect.startswith("pylint."):
+                        sect = sect[len("pylint.") :]
+                    if not sect.isupper() and values:
+                        parser._sections[sect.upper()] = values
 
         if not verbose:
             return
@@ -752,7 +786,6 @@ class OptionsManagerMixIn:
                 try:
                     self.global_set_option(option, value)
                 except (KeyError, optparse.OptionError):
-                    # TODO handle here undeclared options appearing in the config file
                     continue
 
     def load_configuration(self, **kwargs):
@@ -800,6 +833,10 @@ class OptionsManagerMixIn:
         with _patch_optparse():
             return self.cmdline_parser.format_help()
 
+    def helpfunc(self, option, opt, val, p, level):  # pylint: disable=unused-argument
+        print(self.help(level))
+        sys.exit(0)
+
 
 class OptionsProviderMixIn:
     """Mixin to provide options to an OptionsManager"""
@@ -807,7 +844,7 @@ class OptionsProviderMixIn:
     # those attributes should be overridden
     priority = -1
     name = "default"
-    options = ()
+    options = ()  # type: Tuple[Tuple[str, Dict[str, Any]], ...]
     level = 0
 
     def __init__(self):
